package com.qmth.cqb.paper.model;

import cn.com.qmth.examcloud.common.dto.question.enums.QuesStructType;
import com.qmth.cqb.question.model.Question;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.annotation.Id;
import org.springframework.data.mongodb.core.mapping.DBRef;

import java.io.Serializable;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

public class PaperDetailUnit implements Serializable, Comparable<PaperDetailUnit> {

    private static final long serialVersionUID = -8854150484922002075L;

    public static DecimalFormat SCORE_FORMATER = new DecimalFormat("####.#");

    @Id
    private String id;

    @DBRef
    private Paper paper;// 关联的试卷

    private Integer number;// 小题序号

    private Double score;// 小题分数

    private String subScores;//对应套题的情况下，各子题的分数分布

    @DBRef
    private PaperDetail paperDetail;// 关联的大题

    private QuesStructType questionType;// 小题类型

    @DBRef
    private Question question;// 关联试题

    private String creator;// 创建人id

    private String createTime;// 创建时间

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public Paper getPaper() {
        return paper;
    }

    public void setPaper(Paper paper) {
        this.paper = paper;
    }

    public Integer getNumber() {
        return number;
    }

    public void setNumber(Integer number) {
        this.number = number;
    }

    public Double getScore() {
        return score;
    }

    public void setScore(Double score) {
        this.score = score;
        if (question != null && question.getSubQuestions() != null) {
            initSubScores(score, question.getSubQuestions());
        }
    }

    public PaperDetail getPaperDetail() {
        return paperDetail;
    }

    public void setPaperDetail(PaperDetail paperDetail) {
        this.paperDetail = paperDetail;
    }

    public QuesStructType getQuestionType() {
        return questionType;
    }

    public void setQuestionType(QuesStructType questionType) {
        this.questionType = questionType;
    }

    public Question getQuestion() {
        return question;
    }

    public void setQuestion(Question question) {
        this.question = question;
        if (score != null && question != null && question.getSubQuestions() != null) {
            initSubScores(score, question.getSubQuestions());
        }
    }

    public String getCreator() {
        return creator;
    }

    public void setCreator(String creator) {
        this.creator = creator;
    }

    public String getCreateTime() {
        return createTime;
    }

    public void setCreateTime(String createTime) {
        this.createTime = createTime;
    }

    public String getSubScores() {
        return subScores;
    }

    public void setSubScores(String subScores) {
        this.subScores = subScores;
    }

    public PaperDetailUnit() {
    }

    @Override
    public int compareTo(PaperDetailUnit unit) {
        if (unit != null) {
            if (this.getQuestionType().getId() > unit.getQuestionType().getId()) {
                return 1;
            } else if (this.getQuestionType().getId() == unit.getQuestionType().getId()) {
                return 0;
            }
        }
        return -1;
    }

    /**
     * 根据预设总分自动拆解套题子题的分数分布
     * 目前采取均分后累加到最后一题的做法
     *
     * @param totalScore
     * @param questionList
     */
    private void initSubScores(double totalScore, List<Question> questionList) {
        List<Double> scoreList = new ArrayList<>();
        int count = questionList.size();
        if (count > 0) {
            int baseScore = (int) (totalScore / count);
            double leftScore = totalScore;
            for (int i = 0; i < count; i++) {
                scoreList.add((double) baseScore);
                leftScore -= baseScore;
            }
            if (leftScore > 0) {
                scoreList.set(count - 1, baseScore + leftScore);
            }
            updateSubScore(scoreList);
        }
    }

    public List<Double> getSubScoreList() {
        List<Double> list = new ArrayList<>();
        String[] values = StringUtils.split(StringUtils.trimToEmpty(subScores), ",");
        for (String value : values) {
            try {
                list.add(Double.valueOf(value));
            } catch (Exception e) {

            }
        }
        return list;
    }

    public void updateSubScore(List<Double> list) {
        double totalScore = 0;
        if (list != null) {
            List<String> values = new LinkedList<>();
            for (Double value : list) {
                values.add(SCORE_FORMATER.format(value));
                totalScore += value;
            }
            setSubScores(StringUtils.join(values, ","));
        } else {
            setSubScores(null);
        }
        this.score = totalScore;
    }
}
